/**
 * Copyright 2004-2012 Crypto-Pro. All rights reserved.
 * Этот файл содержит информацию, являющуюся
 * собственностью компании Крипто-Про.
 *
 * Любая часть этого файла не может быть скопирована,
 * исправлена, переведена на другие языки,
 * локализована или модифицирована любым способом,
 * откомпилирована, передана по сети с или на
 * любую компьютерную систему без предварительного
 * заключения соглашения с компанией Крипто-Про.
 */
package CAdES;

import org.bouncycastle.tsp.TimeStampToken;
import ru.CryptoPro.CAdES.CAdESSignature;
import ru.CryptoPro.CAdES.CAdESSigner;
import ru.CryptoPro.CAdES.CAdESType;
import ru.CryptoPro.CAdES.exception.CAdESException;
import ru.CryptoPro.JCP.JCP;

import java.io.FileInputStream;
import java.io.IOException;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

/**
 * Различные константы для примеров.
 * 
 * @author Yevgeniy, 17/04/2012
 *
 */
public class Configuration {

	/**
	 * Алиас ключа.
	 */
	private static final String KEY_ALIAS = "gost_exch";
	/**
	 * Пароль к ключу.
	 */
	private static final char[] KEY_PASSWORD = "Pass1234".toCharArray();
	/**
	 * Данные для подписи.
	 */
	public static final byte[] DATA = "Security is only our business.".getBytes();
	/**
	 * Место хранения файлов.
	 */
	public static final String TEST_DIR = "C:\\TESTS\\";
	/**
	 * Имя файла с подписью для сохранения.
	 */
	public static final String SIGNATURE_FILENAME = "CMSSignature.bin";
	/**
	 * Отступ при печати.
	 */
	private static final String TAG = "***";
	/**
	 * Имя файла с CRL для проверки подписи CAdES-BES. Можно быть null.
	 */
	public static final String CRL_FILENAME = "C:\\TESTS\\CRLS\\certcrl.crl";
	/**
	 * Адрес службы штампов.
	 */
	public static final String TSA_ADDRESS = "http://www.cryptopro.ru:80/tsp/";
	
	/**
	 * Загрузка закрытого ключа и цепочки сертификатов из пользовательского
	 * контейнера.
     *
	 * @param chain Цепочка сертификатов. Заполнится после загрузки хранилища.
	 * @return закрытый ключ из контейнера.
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws IOException
	 * @throws UnrecoverableKeyException
	 */
	public static PrivateKey loadConfiguration(Collection<X509Certificate> chain) 
		throws KeyStoreException, NoSuchAlgorithmException, CertificateException, 
		IOException, UnrecoverableKeyException {
		
		return loadConfiguration(JCP.HD_STORE_NAME, null, null, Configuration.KEY_ALIAS, 
			Configuration.KEY_PASSWORD, chain);
	}
	
	/**
	 * Загрузка закрытого ключа и цепочки сертификатов из пользовательского
	 * контейнера.
     *
     * @param storeType Тип хранилища.
     * @param storeFile Путь к хранилищу.
     * @param storePassword Пароль хранилища.
     * @param alias Идентификатор ключа.
     * @param password Пароль ключа.
	 * @param chain Цепочка сертификатов. Заполнится после загрузки хранилища.
	 * @return закрытый ключ из контейнера.
	 * @throws KeyStoreException
	 * @throws NoSuchAlgorithmException
	 * @throws CertificateException
	 * @throws IOException
	 * @throws UnrecoverableKeyException
	 */
	public static PrivateKey loadConfiguration(String storeType, String storeFile,
		char[] storePassword, String alias, char[] password, Collection<X509Certificate> 
		chain) throws KeyStoreException, NoSuchAlgorithmException, CertificateException, 
		IOException, UnrecoverableKeyException {
		
		KeyStore keyStore = KeyStore.getInstance(storeType);
		keyStore.load(storeFile == null ? null : new FileInputStream(storeFile), 
			storePassword);
		
		PrivateKey privateKey = 
			(PrivateKey) keyStore.getKey(alias, password);

		// Получаем цепочку сертификатов. 
		List<Certificate> lChain = 
			Arrays.asList(keyStore.getCertificateChain(alias));
	
		// Конвертируем цепочку в X509Certificate.
		Collection<X509Certificate> xChain = 
			Arrays.asList((lChain).toArray(new X509Certificate[lChain.size()]));
		
		chain.addAll(xChain);
		
		return privateKey;
	}
	
	/**
	 * Вывод информации об отдельной подписи.
	 * 
	 * @param signer Подпись.
	 * @param index Индекс подписи.
	 * @param tab Отступ для удобства печати.
	 */
	private static void printSignerInfo(CAdESSigner signer, int index, String tab) {
		
		X509Certificate signerCert = signer.getSignerCertificate();
		
		System.out.println(tab + " Signature #" + index + " (" + 
			CAdESType.getSignatureTypeName(signer.getSignatureType()) + ")" + 
			(signerCert != null ? (" verified by " + signerCert.getSubjectDN()) : "" ));
						
		if ( signer.getSignatureType() == CAdESType.CAdES_X_Long_Type_1 ) {
							
			TimeStampToken signatureTimeStamp = signer.getSignatureTimestampToken();
			TimeStampToken cadesCTimeStamp = signer.getCAdESCTimestampToken();
			
			if (signatureTimeStamp != null) {
				System.out.println(tab + TAG + " Signature timestamp set: " + 
					signatureTimeStamp.getTimeStampInfo().getGenTime());
			}
			
			if (cadesCTimeStamp != null) {
				System.out.println(tab + TAG + " CAdES-C timestamp set: " + 
					cadesCTimeStamp.getTimeStampInfo().getGenTime());
			}
		}
		
		printCountersignerInfos(signer.getCAdESCountersignerInfos());
	}
	
	/**
	 * Вывод информации о заверителях отдельной подписи.
	 * 
	 * @param countersigners Список заверителей.
	 */
	private static void printCountersignerInfos(CAdESSigner[] countersigners) {
		
		// Заверяющие подписи.
		int countersignerIndex = 1;
		for (CAdESSigner countersigner : countersigners) {
			printSignerInfo(countersigner, countersignerIndex++, TAG);
		}
	}
	
	/**
	 * Вывод информации о подписи: кто подписал, тип подписи, штампы времени.
	 * 
	 * @param signature CAdES подпись.
	 */
	public static void printSignatureInfo(CAdESSignature signature) {
		
		// Список подписей.
		int signerIndex = 1;
		for (CAdESSigner signer : signature.getCAdESSignerInfos()) {
			printSignerInfo(signer, signerIndex++, "");
		}
	}

    /**
     * Выводим информацию о случившейся ошибке. Для CAdESException дополнительно
     * выводится код ошибки.
     *
     * @param e Исключение.
     */
    public static void printCAdESException(Exception e) {

        if (e instanceof CAdESException) {
            System.out.println(e.getMessage() + " (" + ((CAdESException)e).getErrorCode() + ")");
        } else if (e.getCause() instanceof CAdESException) {
            CAdESException ex = (CAdESException)e.getCause();
            System.out.println(ex.getMessage() + " (" + ex.getErrorCode() + ")");
        } else {
            e.printStackTrace();
        }

    }
}
